<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Acl.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Controller_Plugin_Abstract
 */
require_once 'Zend/Controller/Plugin/Abstract.php';

/**
 * Plugin for managing access control lists
 *  
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Controller_Plugin_Acl extends Zend_Controller_Plugin_Abstract
{

    /**
     * Get access lists
     * 
     * @return void
     */
    public function getAcl ()
    {
        $acl = new Giscuit_Acl();
        $acl->setRoles();
        $acl->setRules();
        $acl->allow('admin');
        
        return $acl;
    }

    /**
     * Check if current user is allowed to access specified controller and action
     * 
     * @return boolean
     */
    public function check ()
    {
        if (Zend_Registry::isRegistered('acl')) {
            $acl = Zend_Registry::get('acl');
        } else {
            
            //Cache
            if (Zend_Registry::isRegistered('cache')) {
                $cache = Zend_Registry::get('cache');
                $cacheId = md5(GT_APP_PATH) . '_acl';
                
                if (! $cache->test($cacheId)) {
                    $acl = $this->getAcl();
                    $cache->save($acl, $cacheId);
                } else {
                    $acl = $cache->load($cacheId);
                }
            } else {
                $acl = $this->getAcl();
            }
            
            Zend_Registry::set('acl', $acl);
        }
        
        $auth = Zend_Auth::getInstance();
        
        if ($auth->hasIdentity() && ! empty($auth->getIdentity()->id)) {
            $user = $auth->getIdentity();
        } else {
            $user = new ArrayObject();
            $user->group = 'guest';
        }
        
        $request = $this->getRequest();
        
        try {
            $resource = $request->getModuleName() . '_' . $request->getControllerName() . '_controller';
            return $acl->isAllowed($user->group, $resource, $request->getActionName());
        } catch (Zend_Acl_Exception $e) {
            return false;
        }
    
    }

    /**
     * Check if current user is allowed to access specified controller and action.
     * This is ran during Zend_Controller_Front routeShutdown event.
     * 
     * @return void
     */
    public function routeShutdown (Zend_Controller_Request_Abstract $request)
    {
        //Check user
        $allowed = $this->check();
   
        //Check
        if ($allowed) {
            return;
        }
        
        $response = Zend_Controller_Front::getInstance()->getResponse();
        $response->clearAllHeaders();
        
        //Forbidden
        header('HTTP/1.0 401 Unauthorized');
        
        if($request->getParam('module') == 'default' && $request->getParam('controller') == 'draw') {
            $translate = Zend_Registry::get('Zend_Translate');
            Giscuit_Map_Map::drawError($translate->_('accessForbidden'));   
        } else {
        
            $view = new Zend_View();
            $view->addScriptPath(GT_MODULES_PATH . DIR_SEP . 'default' . DIR_SEP . 
            'views' . DIR_SEP . 'scripts' . DIR_SEP . 'auth');
            echo $view->render('error' . GT_SCRIPT_EXT);
        
        }
        
        exit;
    }

    /**
     * Authenticate
     * 
     * @param array $params Array  account and password 
     * @param array $options <ul>Available options:
     * <li>md5Credential bool - Is password already encrypted with md5</li>
     * <li>nonPersistent bool - Set session data storage type to non persistent</li></ul>
     * @return string Error key or empty if no error occurred
     */
    public static function authenticate (array $params, array $options = array())
    {
        
        $db = Zend_Registry::get('db');
        $config = Zend_Registry::get('config');
        $auth = Zend_Auth::getInstance();
        
        $authAdapter = new Zend_Auth_Adapter_DbTable($db);
        
        $authAdapter->setTableName($config->database->schema . '.users');
        
        $authAdapter->setIdentityColumn('account');
        $authAdapter->setIdentity(strtolower($params['account']));
        
        $authAdapter->setCredentialColumn('password');
        
        if (isset($options['md5Credential']) && $options['md5Credential'] == true) {
            $authAdapter->setCredential($params['password']);    
        } else {
            $authAdapter->setCredential($params['password'] . $config->salt);
            $authAdapter->setCredentialTreatment("md5(?)");
        }
        
        //Non persistent
        if (isset($options['nonPersistent']) && $options['nonPersistent'] == true) {     
            $storage = new Zend_Auth_Storage_NonPersistent();
            $auth->setStorage($storage);
        }
        
        $authResult = $auth->authenticate($authAdapter);
        
        $error = NULL;
        
        if (! $authResult->isValid()) {
            $error = 'InvalidCredentials';
        } else {
            
            $data = $authAdapter->getResultRowObject(array(
                'account' , 
                'fname' , 
                'lname' , 
                'active' , 
                'group' , 
                'language' , 
                'expiration_date' , 
                'ip' , 
                'id' ,
                'email'
            ));
            
            if (! empty($data->expiration_date) && strtotime($data->expiration_date) < time()) {
                $error = 'IdExpired';
            }
            
            if (trim($data->ip) != '' && $data->ip != $_SERVER['REMOTE_ADDR']) {
                $error = 'InvalidIp';
            }
            
            if ($data->active == 'f') {
                $error = 'IdNotActivated';
            }
        
        }
        
        if (! $error) {
            $auth->getStorage()->write($data);
            
            if (isset($options['nonPersistent']) && $options['nonPersistent'] == true) {
                $settings = new stdClass;
                Zend_Registry::set('settings', $settings);
            } else {
                $usersTable = new Giscuit_Db_Table_Users();
                $usersTable->updateLastLoginDate($auth->getIdentity()->id);
                
                $sessionTable = Zend_Registry::get('sessionTable');
                $where = $sessionTable->getAdapter()->quoteInto('user_id = ?', $auth->getIdentity()->id);
                $sessionTable->delete($where);
                            
                $settings = Zend_Registry::get('settings');
                
                Zend_Registry::get('Zend_Translate')->setLocale($auth->getIdentity()->language);
            }
        
        }
        
        return $error;
    }
    
}