<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Articles.php 2897 2012-08-02 11:41:08Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Multilang
 */
require_once 'Giscuit/Db/Table/Multilang.php';

/**
 * Abstract class for a multilang table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
abstract class Giscuit_Db_Table_Articles extends Giscuit_Db_Table_Multilang
{

    /**
     * Names of multilang fields inside the table
     * 
     * @var array
     */
    protected $_multilangTextColumns = array('title', 'tags', 'desc', 'text');

    /**
     * Get items
     *   
     * @param array $options <ul>Available options:
     * <li>withDesc bool - Include desc field in the results</li>
     * <li>withText bool - Include text field in the results</li>
     * <li>withTitle bool - Include title field in the results</li>
     * <li>withTags bool - Include tags field in the results</li>
     * <li>withMultilangFields bool - Add multilanguage fields based on current language</li>
     * <li>whereCurrentUser bool - Get only elements that belong to current user</li>
     * <li>whereActive bool - Get only active elements</li>
     * <li>whereId integer - Fetch rows that match the id</li>
     * <li>limit integer - Maximum number of rows to return</li>
     * <li>limitStart integer - Offset of the first row to return</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getItems (array $options)
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        //Set fields
        $fields = array(
            'id' , 
            'active' , 
            'created_date' => "to_char({$this->_name}.created_date, '" . GT_DATETIME_STYLE_DB . "')" , 
            'image' , 
            'views' , 
            'comm_count' => "(SELECT count(comm.id) FROM {$this->_schema}.comm WHERE @{$this->_name}.id = comm.part)"
        );
        $fields = array_merge($fields, $this->_getOptionalFields());
        
        $select->from($this, $fields);
        
        //Join users table
        $select->joinLeft(array(
            'users' => $this->_schema . '.users'
        ), "users.id = {$this->_name}.owner_id", array(
            'owner_name' => "(users.fname||' '||users.lname)"
        ));
        
        //Order
        $select->order($this->_name . '.created_date desc');
        
        //Fetch desc
        if (! empty($options['withDesc'])) {
            $this->joinMultilangField($select, 'desc');
        }
        
        //Fetch text
        if (! empty($options['withText'])) {
            $this->joinMultilangField($select, 'text');
        }
        
        //Fetch title
        if (! empty($options['withTitle'])) {
            $this->joinMultilangField($select, 'title');
        }
        
        //Fetch tags
        if (! empty($options['withTags'])) {
            $this->joinMultilangField($select, 'tags');
        }
     
        //Limit
        if (! empty($options['limit'])) {
            if (empty($options['limitStart'])) {
                $options['limitStart'] = 0;
            }
            $select->limit($options['limit'], $options['limitStart']);
        }
        
        $this->whereCurrentUser($select, $options);
        $this->whereActive($select, $options);
        $this->whereId($select, $options);
        $this->withMultilangFields($select, $options);
        
        $result = $this->fetchAll($select);

        //Filter
        $htmlSpecialChars = new Giscuit_Filter_HtmlSpecialChars();
        $htmlPurifier = new Giscuit_Filter_HTMLPurifier();
        
        foreach ($result as $item) {
            if (! empty($options['withTitle'])) {
                $item->title = $htmlSpecialChars->filter($item->title);
                if(empty($item->title)) { 
                    $item->title = $translate->_('emptyContent');
                } 
            }

            if (! empty($options['withTags'])) {
                $item->tags = $htmlSpecialChars->filter($item->tags);
            }
            
            if (! empty($options['withDesc'])) {
                $item->desc = $htmlSpecialChars->filter($item->desc);
                
                if(empty($item->desc)) { 
                    $item->desc = $translate->_('emptyContent');
                }
            }
            
            
            if (! empty($options['withText'])) {
                $item->text = $htmlPurifier->filter($item->text);
                
                if(empty($item->text)) { 
                    $item->text = $translate->_('emptyContent');
                }
            }
            
        }
        
        return $result;
    }

    /**
     * Add item
     * 
     * @param Zend_Form $data A form object populated with request data
     * @return integer The id of the new element
     */
    public function addItem ($data)
    {
        $newRow = $this->createRow();
        
        $this->manageMultilangText($this->getMultilangPostData($data->getValues()), $newRow);
        
        //Active
        $newRow->active = $data->getValue('active');
        
        //Owner
        $newRow->owner_id = Zend_Auth::getInstance()->getIdentity()->id;
        
        //Date
        $newRow->created_date = new Zend_Db_Expr('now()');
        
        $this->_manageAddItem($newRow, $data);
        
        //Save
        return $newRow->save();
    }

    /**
     * Update item
     * 
     * @param integer $id The id of the row to edit 
     * @param Zend_Form $data A form object populated with request data
     * @return void
     */
    public function editItem ($id, $data)
    {
        $rows = $this->find($id);
        $row = $rows->current();
        
        //Multilang text
        $this->manageMultilangText($this->getMultilangPostData($data->getValues()), $row, Giscuit_Db_Table_Multilang::UPDATE);
        
        //Set new values
        $row->active = $data->getValue('active');
        
        $row->created_date = $data->getValue('created_date');
        
        //Call filters
        $data->__toString();
        
        $this->_manageEditItem($row, $data);
        
        //Save
        $row->save();
    }
    
    /**
     * Increment table views column by one
     * 
     * @param Zend_Db_Table_Row $row
     * @return void
     */
    public function incrementViews ($row)
    {
        $settings = Zend_Registry::get('settings');
        
        $viewSessionName = $this->_name . '_views_' . $row->id;
        
        if (! $settings->$viewSessionName) {
            $row->views += 1;
            
            $data = array(
                'views' => $row->views
            );
            $where = $this->getAdapter()->quoteInto('id = ?', $row->id);
            $this->update($data, $where);
            $settings->$viewSessionName = true;
        }
    }

}