<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: GeometryTables.php 3395 2013-05-03 09:14:19Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for geometry tables table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_GeometryTables extends Giscuit_Db_Table_Abstract
{

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'geometry_columns';

    /**
     * Table primary key name
     * 
     * @var string
     */
    protected $_primary = 'f_table_name';

    /**
     * Table schema name
     * 
     * @var string
     */
    protected $_schema = 'public';
    
    /**
     * Table column types
     * 
     * @var array
     */
    public $columnTypes = array(
        'int2' => 'smallint' , 
        'int4' => 'integer' , 
        'int8' => 'bigint' , 
        'float4' => 'real' , 
        'float8' => 'double precision' , 
        'numeric' => 'numeric' , 
        'varchar' => 'character varying' , 
        'bpchar' => 'character' , 
        'text' => 'text' , 
        'bytea' => 'bytea' , 
        'timestamp' => 'timestamp without time zone' , 
        'timestamptz' => 'timestamp with time zone' , 
        'interval' => 'interval' , 
        'date' => 'date' , 
        'time' => 'time without time zone' , 
        'timetz' => 'time with time zone' , 
        'bool' => 'boolean' , 
        'geometry' => 'geometry'
    );

    /**
     * Get all database tables
     * 
     * @param array $options Available options:
     * <ul><li>schema string - Database schema</li>
     * <li>whereTables string - Get results that match this comma separated list of tables</li></ul> 
     * @return array All tables
     */
    public function getTables (array $options = array())
    {
        $config = Zend_Registry::get('config');
        
        if(! isset($options['schema'])) {
            $schema = $config->database->schema . '_layers';
        } else {
            $schema = $options['schema'];
        }
        
        //Tables
        $tablesSelect = $this->getAdapter()->select();
        $tablesSelect->from('pg_tables', array(
            'schemaname',
            'tablename',
            'is_view' => new Zend_Db_Expr("'f'")
        ));
        
        $tablesSelect->joinLeft(array(
            $this->_name
        ), "schemaname = {$this->_name}.f_table_schema AND tablename = {$this->_name}.f_table_name", array(
            'type' ,
            'f_geometry_column' ,
            'srid'
        ));
        
        $tablesSelect->where('schemaname = ?', $schema);
        
        //whereTables
        if (! empty($options['whereTables'])) {
             $tablesSelect->where("'{$options['whereTables']}' LIKE '%' || tablename || ',%'");
        }
        
        //whereName
        if (! empty($options['whereName'])) {
            $tablesSelect->where('tablename = ?', $options['whereName']);
        }
        
        //whereGeometryTable
        if (! empty($options['whereGeometryTable'])) {
            $tablesSelect->where('type IS NOT NULL');
        }

        //Views
        $viewsSelect = $this->getAdapter()->select();
        $viewsSelect->from('pg_views', array(
            'schemaname',
            'tablename' => 'viewname',
            'is_view' => new Zend_Db_Expr("'t'")
        ));
        
        $viewsSelect->joinLeft(array(
            $this->_name
        ), "schemaname = {$this->_name}.f_table_schema AND viewname = {$this->_name}.f_table_name", array(
            'type' ,
            'f_geometry_column' ,
            'srid'
        ));
        
        $viewsSelect->where('schemaname = ?', $schema);
        
        //whereTables
        if (! empty($options['whereTables'])) {
            $viewsSelect->where("'{$options['whereTables']}' LIKE '%' || viewname || ',%'");
        }
        
        //whereName
        if (! empty($options['whereName'])) {
            $viewsSelect->where('viewname = ?', $options['whereName']);
        }
                
        //Union
        $select = $this->getAdapter()->select();
        $select->union(array($viewsSelect, $tablesSelect));
        $select->order("tablename");
        
        //Result
        $result = $this->getAdapter()->fetchAll($select, array(), Zend_Db::FETCH_ASSOC);
        foreach ($result as $key => $value) {
            $value['f_table_name'] = $value['tablename'];
            $result[$value['tablename']] = $value;
            unset($result[$key]);
        }
                
        return $result;
    }
    
    /**
     * Get all spatial reference systems
     * 
     * @param array $options 
     * @return array
     */
    public function getAllSpatialRefSys (array $options = array())
    {

        $select = $this->getAdapter()->select();
        $select->from('public.spatial_ref_sys', array(
            'srid' ,
            'name' => 'srtext'
        ));
        
        $this->getAdapter()->setFetchMode(Zend_Db::FETCH_ASSOC);
        $result = $this->getAdapter()->fetchAll($select);
        $this->getAdapter()->setFetchMode(Zend_Db::FETCH_OBJ);

        foreach($result as $key => $row) {
            //
            $name = $row['name'];
            
            //
            if(! $name) {
                unset($result[$key]);
                continue;
            }
            
            //
            $start = strpos($name, 'PROJCS');
            if($start === false) {
                $start = strpos($name, 'GEOGCS');
            }
            $start += 8;
            
            //
            $end = strpos($name, '"', $start);
            
            //
            $row['name'] = substr($name, $start, $end - $start) . " (SRID: {$row['srid']})";
            $result[$key] = $row;
        }
                
        return $result;
    }
    
    /**
     * Get projection by srid
     * 
     * @param integer $srid Srid value
     * @return Zend_Db_Table_Row
     */
    public function getProjTextBySrid ($srid)
    {

        $select = $this->getAdapter()->select();
        $select->from('public.spatial_ref_sys', array(
            'proj4text' 
        ));
        $select->where($this->getAdapter()->quoteIdentifier('spatial_ref_sys.srid') . ' = ?', $srid);
        
        $result = $this->getAdapter()->fetchRow($select);
                        
        return $result;
    }
    
    /**
     * Delete specified table
     * 
     * @param string $tableName Name of the table
     * @param string $schema Database schema
     * @return void
     */
    public function deleteTable($tableName, $schema = NULL)
    {
        $config = Zend_Registry::get('config');
        
        if($schema == NULL) {
            $schema = $config->database->schema . '_layers';
        }
        
        $this->getAdapter()->query("DROP TABLE  IF EXISTS \"$schema\".\"$tableName\" CASCADE");
        
        //Delete record from geometry_columns table
        $this->delete("f_table_name = '$tableName' AND f_table_schema = '$schema'");
        
    }
    
    /**
     * Get current PostGIS version
     * 
     * @return string
     */
    public static function getPostgisVersion()
    {
        $db = Zend_Registry::get('db');
    
        $fullVersion = $db->fetchRow("SELECT postgis_version()");
        $fullVersion = explode(' ', $fullVersion->postgis_version);
        
        return $fullVersion[0];
    }
}
