<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: History.php 3010 2012-10-03 12:02:59Z vlad.bruma $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for history table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_History extends Giscuit_Db_Table_Abstract
{
    /**
     * Table primary key name
     * 
     * @var string
     */
    protected $_primary = 'id';
    
    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'history';
    
    /**
     * Get item
     * 
     * @param integer $id Id of the required element
     * @param array $options Available options:
     * <ul><li>withGidlist bool - Include gidlist field in the result</li>
     * <li>whereFetchUserId integer - Get only elements with given fetch_user_id</li></ul>
     * @return Zend_Db_Table_Row
     */
    public function getItem ($id, $options = NULL)
    {
        $select = $this->select();
        $select->setIntegrityCheck(false);
        $fields = array(
            'id' , 
            'fetch_user_id' , 
            'fetch_date' =>  new Zend_Db_Expr("extract(epoch from {$this->_name}.fetch_date)"),
            'ext' ,
            'active' 
        );
        
        //Gidlist
        if(isset($options['withGidlist']) && $options['withGidlist'] == true) {
            $fields = array_merge($fields, array('gidlist'));
        }
        
        $select->from($this, $fields);
        $select->where($this->_name . '.id = ?', $id);
        
        if(isset($options['whereFetchUserId'])) {
            $select->where($this->_name . '.fetch_user_id = ?', $options['whereFetchUserId']);
        }
        
        //Result
        $result = $this->fetchRow($select);
        
        if(isset($options['withGidlist']) && $options['withGidlist'] == true && isset($result->gidlist)) {
            $result->gidlist = unserialize($result->gidlist);
        }
        
        return $result;
    }
    
    /**
     * Set active field
     * 
     * @param integer $id Id of the required element
     * @param bool $bool If true sets active, false sets inactive
     * @return void
     */
    public function setActive($id, $bool = false)
    {
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        $rows = $this->find($id);
          $row = $rows->current();

        if($bool == true) {
            $row->active = 't';
            
            $row->activate_user_id = $userId;
            $row->activate_date = new Zend_Db_Expr('now()');
        } else {
            $row->active = 'f'; 
        }

        
          
          $row->save();
    }
    
    /**
     * Set value for gidlist
     * 
     * @param integer $id Id of the required element
     * @param string $value Gidlist value
     * @return void
     */
    public function setGidlist($id, $value)
    {
        $rows = $this->find($id);
          $row = $rows->current();

          $row->gidlist = serialize($value);
          
          $row->save();
    }
    
    /**
     * Add new item
     * 
     * @param Zend_Form $data Zend Form history data
     * @param array $options <ul>Available options:
     * <li>active t,f - Activate the new entry or not</li>
     * <li>withOutActivateUserId bool - activate_user_id field value</li></ul>
     * @return integer The id of the new element
     */
    public function addItem($data, array $options = array())
    {
        if($data instanceof Zend_Form) {
            $data = $data->getValues();
        }

        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        $newRow = $this->createRow();
        
        if(! empty($data['comment'])) {
            $newRow->comment = $data['comment'];
        }
        
        if(! empty($data['ext'])) {
            $newRow->ext = $data['ext'];
        }
        
        if(! empty($options['active'])) {
            $newRow->active = $options['active'];
            
            if($options['active'] == 't') {
                $newRow->activate_date = new Zend_Db_Expr('now()');
                
                if(isset($options['withOutActivateUserId']) && $options['withOutActivateUserId'] == true) {
                    $newRow->activate_user_id = $userId;
                }
            }
        } else {
            $newRow->active = 'f';
        }
        
        $newRow->fetch_date = new Zend_Db_Expr('now()');
        $newRow->fetch_user_id = $userId;
        
        $newRow->save();
        
        return $newRow->id;
    }
    
    /**
     * Get inactive items
     * 
     * @return Zend_Db_Table_Rowset
     */
    public function getInactiveItems()
    {
        $config = Zend_Registry::get('config');
        
        $schema = $config->database->schema;
        
        $select = $this->select();
        $select->setIntegrityCheck(false);
        $fields = array(
            'id' , 
            'gidlist' ,
            'upload_date' => new Zend_Db_Expr("to_char({$this->_name}.upload_date, '" . GT_DATETIME_STYLE_DB . "')") ,
            'upload_extent' => new Zend_Db_Expr("{$this->_name}.upload_extent::box2d")
        );
        $select->from($this, $fields);
        $select->where($this->_name . '.active = ?', 'f');
        
        $select->where($this->_name . '.upload_date IS NOT NULL');
        
        $select->order($this->_name . ".upload_date desc");
                
        $select->joinLeft(array(
            'users' => $schema . '.users'
        ), "users.id = {$this->_name}.fetch_user_id", array('user_name' => "(users.fname||' '||users.lname)"));
          
        //Result
        $result = $this->fetchAll($select);
        
        foreach($result as $row) {
            $row->gidlist = unserialize($row->gidlist);
        }
               
        return $result;
    }

    /**
     * Update item
     * 
     * @param integer $id Id of the required element
     * @param array $options <ul>Available options:
     * <li>bounds array - Extent bounds</li></ul>
     * @return void
     */
    public function ediItem ($id, array $options = array())
    {
        $config = Zend_Registry::get('config');
        
        $rows = $this->find($id);
        $row = $rows->current();
        
        if (! empty($options['bounds'])) {
            $bounds = $options['bounds'];
            
            $uploadExtent = "ST_SetSRID(ST_Buffer('BOX({$bounds['minx']} {$bounds['miny']}, {$bounds['maxx']} {$bounds['maxy']})'::box2d, 0), {$config->map->srid})";
            $row->upload_extent = new Zend_Db_Expr($uploadExtent);
            
            $row->upload_date = new Zend_Db_Expr('now()');
        }
        
        $row->save();
    }
    
    /**
     * Check if current user can activate package
     * 
     * @param integer $id The id of the row to check
     * @param array $options 
     * @return boolean
     */
    public function isAllowed($id, array $options = array())
    {
        //Get current user id
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        if (empty($id)) {
            throw new Giscuit_Exception('Undefined package id');
            return false;
        }
        
        $result = $this->getItem($id, array(
            'whereUserId' => $userId , 
            'withGidlist' => true
        ));
        
        if (empty($result)) {
            throw new Giscuit_Exception("Package does not exist");
            return false;
        }
        
        if ($result->active == 't') {
            throw new Giscuit_Exception("Package already activated");
            return false;
        }
        
        //Get editable layer names
        $layers = new Giscuit_Db_Table_Layers();
        
        $layers = new Giscuit_Db_Table_Layers();
        $tables = $layers->getEditTables(array(
            'whereHistory' => true
        ));
        
        $editLayers = array();
        foreach ($tables as $table) {
            $editLayers[] = $table->info('name');
        }
        
        //Check if the user can activate the layers in the package
        $layers = array_keys($result->gidlist);
        foreach ($layers as $layer) {
            if (! in_array($layer, $editLayers)) {
                throw new Giscuit_Exception("Package contains layer that you don't have permission to edit");
                return false;
            }
        }
        
        return true;
    }

}
