<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Mapstats.php 1133 2011-03-14 16:12:16Z ue $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for mapstats table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_Mapstats extends Giscuit_Db_Table_Abstract
{

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'mapstats';
    
    /**
     * Get service requests totals
     * 
     * @param array $options Available options:
     * <ul><li>whereServices array - Fetch rows that match the services</li>
     * <li>whereAccesses array - Fetch rows that match the accesses</li>
     * <li>whereDateInterval array - Get elements between the start time and end time</li>
     * <li>whereAccountLike string - Get elements with users.account like the passed string</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getTotals(array $options = array()) 
    {
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        $fields = array(
            'service' ,
            'total' => new Zend_Db_Expr("count({$this->_name}.id)"), 
            'size' => new Zend_Db_Expr("sum({$this->_name}.size)")
        );
        
        $select->from($this, $fields);

        $this->whereDateInterval($select, $options);
        $this->whereServices($select, $options);
        $this->whereAccesses($select, $options);
        $this->whereAccountLike($select, $options);
        
        $select->group($this->_name . '.service');

        $result = $this->fetchAll($select);
        
        return $result;
    }
    
    
    /**
     * Fetch rows that partially or totally match the users.account field
     * 
     * @param Zend_Db_Select $select
     * @param array $options Available options:
     * <ul><li>whereAccountLike string - If option is set selects rows with account like the value</li></ul>
     * @return void
     */
    public function whereAccountLike ($select, array $options = array())
    {
        if (! isset($options['whereAccountLike'])) {
            return;
        }
        
        $select->joinLeft(array(
            'users' => $this->_schema . '.users' ,
            ), "{$this->_name}.user_id = users.id" ,
            NULL
        );
        
        $select->where("users.account LIKE ?", $options['whereAccountLike']);
    }
    
    /**
     * Fetch rows that match the service field
     * 
     * @param Zend_Db_Select $select
     * @param array $options Available options:
     * <ul><li>whereServices array - If option is set selects rows with the given services</li></ul>
     * @return void
     */
    public function whereServices ($select, array $options = array())
    {
        if (! isset($options['whereServices'])) {
            return;
        }
        
        if (is_array($options['whereServices'])) {
            $whereServices = $this->_name . '.service = ' . implode(" OR {$this->_name}.service  = ", $options['whereServices']);
            $select->where("({$whereServices})");
        } else {
            $select->where($this->_name . '.service = ?', $options['whereServices']);
        }
    }

    /**
     * Fetch rows that are between date fields
     * 
     * @param Zend_Db_Select $select
     * @param array $options Available options:
     * <ul><li>whereDateInterval array - If option is set selects rows between the given dates</li></ul>
     * @return void
     */
    public function whereDateInterval ($select, array $options = array())
    {
        if (! isset($options['whereDateInterval'])) {
            return;
        }
        
        $whereDateInterval = new Zend_Db_Expr("{$this->_name}.date BETWEEN 
            to_timestamp({$options['whereDateInterval']['startTime']}) 
            AND to_timestamp({$options['whereDateInterval']['endTime']})");

        /*
        $whereDateInterval = new Zend_Db_Expr('floor(extract(epoch FROM ' . $this->_name . '.date)) BETWEEN ' . 
            $options['whereDateInterval']['startTime'] . ' AND ' . $options['whereDateInterval']['endTime']);
        */            
        
        $select->where($whereDateInterval);
    }
    
    /**
     * Fetch rows that match the access field
     * 
     * @param Zend_Db_Select $select
     * @param array $options Available options:
     * <ul><li>whereAccesses array - If option is set selects rows with the given accesses</li></ul>
     * @return void
     */
    public function whereAccesses ($select, array $options = array())
    {
        if (! isset($options['whereAccesses'])) {
            return;
        }
        
        if (is_array($options['whereAccesses'])) {

            // Optimization
            if(count($options['whereAccesses']) == 2) {
                return;
            }
            
            $whereAccesses = $this->_name . '.access = ' . implode(" OR {$this->_name}.access  = ", $options['whereAccesses']);
            $select->where("({$whereAccesses})");
        } else {
            $select->where($this->_name . '.access = ?', $options['whereAccesses']);
        }
    }
    
    /**
     * Get data for charts
     * 
     * @param array $options Available options:
     * <ul><li>interval integer - Number of intervals to create</li>
     * <li>whereServices array - Fetch rows that match the services</li>
     * <li>whereAccesses array - Fetch rows that match the accesses</li>
     * <li>whereDateInterval array - Get elements between the start time and end time</li>
     * <li>whereAccountLike string - Get elements with users.account like the passed string</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getChartItems(array $options = array())
    {
        $select = $this->select();
        $select->setIntegrityCheck(false);

        $interval = "floor((extract(epoch FROM {$this->_name}.date) - {$options['whereDateInterval']['startTime']}) / " . 
            "({$options['whereDateInterval']['endTime']} - {$options['whereDateInterval']['startTime']}) * {$options['interval']})";
        
        $fields = array(
            'amount' => new Zend_Db_Expr("count({$this->_name}.id)") , 
            'interval' => new Zend_Db_Expr($interval)
        );
        
        $select->from($this, $fields);
        
        $this->whereDateInterval($select, $options);
        $this->whereServices($select, $options);
        $this->whereAccesses($select, $options);
        
        $this->whereAccountLike($select, $options);

        $select->group('interval');
        $select->order('interval');
        
        $result = $this->fetchAll($select);
        
        return $result;
    }

    /**
     * Add item
     * 
     * @param array $data Statistics data:
     * <ul><li>service integer - Service type</li>
     * <li>access integer - Access type</li>
     * <li>layers string - Requested layers separated by comma</li>
     * <li>size integer - Size of the data requested</li>
     * <li>user_id integer - If option is set saves the id of the requesting user</li>
     * <li>imageSize array - Size in width and height of the image</li>
     * <li>extent array - Extent of the window</li></ul>
     * @return integer Row id 
     */
    public function addItem ($data)
    {
        $config = Zend_Registry::get('config');
        
        $newRow = $this->createRow();
        
        $newRow->service = $data['service'];
        $newRow->access = $data['access'];
        $newRow->date = new Zend_Db_Expr('now()');
        $newRow->layers = $data['layers'];
        
        $newRow->size = $data['size'];
        
        if (! empty($data['user_id'])) {
            $newRow->user_id = $data['user_id'];
        } else {
            $newRow->user_id = NULL;
        }
        
        if(! empty($data['imageSize'])) {
            $newRow->width = $data['imageSize']['width'];
            $newRow->height = $data['imageSize']['height'];
        } else {
            $newRow->width = NULL;
            $newRow->height = NULL;
        }

        $extent = "ST_SetSRID(ST_Buffer('BOX({$data['extent']['min']['x']} {$data['extent']['min']['y']}, {$data['extent']['max']['x']} {$data['extent']['max']['y']})'::box2d, 0), {$config->map->srid})";
        $newRow->extent = new Zend_Db_Expr($extent);
        
        if(! empty($_SERVER['HTTP_USER_AGENT'])) {
            $newRow->useragent = $_SERVER['HTTP_USER_AGENT'];
        } else {
            $newRow->useragent = NULL;
        }
           
        //Ip address
        if (! empty($_SERVER['HTTP_CLIENT_IP'])) {
            $ip = $_SERVER['HTTP_CLIENT_IP'];
        } elseif (! empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
            $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
        } elseif (! empty($_SERVER['REMOTE_ADDR'])) {
            $ip = $_SERVER['REMOTE_ADDR'];
        } else {
            $ip = NULL;
        }
                
        return $newRow->save();
                
    }
    
}