<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Menu.php 2897 2012-08-02 11:41:08Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Multilang
 */
require_once 'Giscuit/Db/Table/Multilang.php';

/**
 * Class for menu table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_Menu extends Giscuit_Db_Table_Multilang
{

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'menu';

    /**
     * Default menu resource
     * 
     * @var string
     */
    public $defaultResource = 'registered';
    
    /**
     * Contains the names of multilang fields inside the table
     * 
     * @var array
     */
    protected $_multilangTextColumns = array(
        'title' , 
        'tags' , 
        'text'
    );

    /**
     * Add item
     * 
     * @param Zend_Form $data A form object populated with request data
     * @param array $options Available options:
     * <ul><li>parent_id integer - Parent menu section id</li></ul>
     * @return void
     */
    public function addItem ($data, $options = array())
    {
        $newRow = $this->createRow();
        
        $multilangPostData = $this->getMultilangPostData($data->getValues());
        $this->manageMultilangText($multilangPostData, $newRow);
        
        $newRow->url = $data->getValue('url');
        $newRow->active = $data->getValue('active');
        $newRow->type = $data->getValue('type');
        
        if(empty($options['parent_id'])) {
            $options['parent_id'] = NULL;
        }
        $newRow->parent_id = $options['parent_id'];
        
        $newRow->resource = $data->getValue('resource');
        $newRow->privilege = $data->getValue('privilege');
        $newRow->target = $data->getValue('target');
        
        $newRow->save();
        
        $newRow->order = $newRow->id;
        $newRow->save();
    }

    /**
     * Get items
     * 
     * @param array $options Available options:
     * <ul><li>withText bool - Include text field in the results</li>
     * <li>withTitle bool - Include title field in the results</li>
     * <li>withDesc bool - Include desc field in the results</li>
     * <li>withTags bool - Include tags field in the results</li>
     * <li>whereActive t,f - Get only active elements</li>
     * <li>whereId integer - Fetch rows that match the id</li>
     * <li>withMultilangFields bool - Add multilanguage fields based on current language</li>
     * <li>buildYuiMenu string - Build yui menu array from results</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getItems ($options = array())
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        $fields = array(
            'id' , 
            'active' ,
            'parent_id' , 
            'type' , 
            'url' , 
            'resource' , 
            'privilege' , 
            'target'
        );
        
        $select->from($this, $fields);
        
        //Fetch text
        if (! empty($options['withText'])) {
            $this->joinMultilangField($select, 'text');
        }
        
        //Fetch title
        if (! empty($options['withTitle'])) {
            $this->joinMultilangField($select, 'title');
        }
        
        //Fetch tags
        if (! empty($options['withTags'])) {
            $this->joinMultilangField($select, 'tags');
        }
        
        $this->whereActive($select, $options);
        $this->whereId($select, $options);
        $this->withMultilangFields($select, $options);
        
        $parentIdField = "COALESCE({$this->_name}.parent_id, 0)";
        $select->order("$parentIdField ASC, {$this->_name}.order ASC");
        
        $result = $this->getAdapter()->fetchAssoc($select);
       
        //Filter
        $htmlSpecialChars = new Giscuit_Filter_HtmlSpecialChars();
        $htmlPurifier = new Giscuit_Filter_HTMLPurifier();
        
        //Acl
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
        
        foreach ($result as $key => $value) {
            //Acl
            if (empty($result[$key]['resource'])) {
                $result[$key]['resource'] = $this->defaultResource;
            }
            
            if ($isAllowed->direct($result[$key]['resource'], $result[$key]['privilege']) == false) {
                unset($result[$key]);
                continue;
            }

            //Filter
            if (! empty($options['withTitle'])) {
                $result[$key]['title'] = $htmlSpecialChars->filter($result[$key]['title']);
                
                if(empty($result[$key]['title'])) {
                    $result[$key]['title'] = $translate->_('nameEmpty');
                }
            } 
            
            if (! empty($options['withDesc'])) {
                $result[$key]['desc'] = $htmlSpecialChars->filter($result[$key]['desc']);
            }
            
            if (! empty($options['withText'])) {
                $result[$key]['text'] = $htmlPurifier->filter($result[$key]['text']);
            }
        }
        
        return $result;
        
    }

    /**
     * Update item
     * 
     * @param integer $id The id of the row to edit 
     * @param Zend_Form $data A form object populated with request data
     * @return void
     */
    public function editItem ($id, $data)
    {
        $rows = $this->find($id);
        $row = $rows->current();
        
        //Multilang text
        $this->manageMultilangText($this->getMultilangPostData($data->getValues()), $row, Giscuit_Db_Table_Multilang::UPDATE);
        
        //Set new values
        $row->url = $data->getValue('url');
        $row->active = $data->getValue('active');
        $row->type = $data->getValue('type');
        
        $row->resource = $data->getValue('resource');
        $row->privilege = $data->getValue('privilege');
        $row->target = $data->getValue('target');
        
        //Save
        $row->save();
    }
    
}
